@@ -39,9 +39,8 @@ class Script(default.Script):
     def __init__(self, app):
         """Creates a new script for the given application."""
         default.Script.__init__(self, app)
-        self._textInserted = False
         self.movementKeys = ["Up", "Down", "Left", "Right", "Page_Up",
-                             "Page_Down", "Home", "End"]
+                   "Page_Down", "Home", "End"]
 
     def _presentTextAtNewCaretPosition(self, event, otherObj=None):
         """Updates braille, magnification, and outputs speech for the
@@ -52,48 +51,39 @@ class Script(default.Script):
         if self.utilities.isDuplicateEvent(event):
             return
 
-        # We will retrieve information from the last object spoken
-        try:
-            oldObj, oldOffset = self.pointOfReference["lastCursorPosition"]
-        except:
-            oldObj = None
-            oldOffset = -1
-
         # Let the default script's normal behavior do its thing
         #
         default.Script._presentTextAtNewCaretPosition(self, event, otherObj)
 
-        textInserted = self._textInserted
-        self._textInserted = False
         # check if the obj was spoken in the default script
         lastKey, mods = self.utilities.lastKeyAndModifiers()
         if lastKey in self.movementKeys:
             # already spoken in default script
             return
-
-        # check if text was inserted before
-        if textInserted:
-            # we will ignore this event because text was spoken in
-            # onTextInserted
-            return
-
         obj = otherObj or event.source
-        offset = obj.queryText().caretOffset
 
-        difChars = offset - oldOffset
-        if obj == oldObj and (difChars == 1 or difChars == -1):
-            # it seems that the caret moved one position, forward or backward
-            # probably the user is typing, we don't want to speak the entire
-            # line
+        if obj.getState().contains(pyatspi.STATE_SINGLE_LINE):
             return
 
-        self.sayLine(obj)
+        # if Tab key is pressed and there is text selected, we must announce
+        # the text selected because probably we are in a parameter list
+        # and we are jumping between the parameters with the tab key. 
+        hasSelection = False
+        if lastKey in ["Tab", "ISO_Left_Tab"]:
+            [text, startOffset, endOffset] = self.utilities.selectedText(obj)
+            hasSelection = endOffset > 0
+
+        if hasSelection:
+            self.sayPhrase(obj, startOffset, endOffset)
+        else:
+            self.sayLine(obj)
+
+        self._saveLastTextPosition(obj)
 
     def onFocus(self, event):
         """Called whenever an object gets focus.  Overridden here so that
-        we can save the current text cursor position when the event is an
-        object:state-changed:focused and source is a pyatspi.ROLE_TEXT
-        Perhaps this should be moved to the default.py???
+        so that we can avoid speaking text when caret moves after new text
+        receives focus and is spoken.
 
         Arguments:
         - event: the Event
@@ -102,15 +92,11 @@ class Script(default.Script):
         # Let the default script's normal behavior do its thing
         #
         default.Script.onFocus(self, event)
-        #
-        if event.type.startswith("object:state-changed:focused") \
-                 and event.source.getRole() == pyatspi.ROLE_TEXT:
-            # probably it was announced, time to save.
-            self._saveLastCursorPosition(event.source, \
-                   event.source.queryText().caretOffset)
+
+        self._saveLastTextPosition(event.source)
 
     def getUtilities(self):
-        """Returns the utilites for this script."""
+        """Returns the utilities for this script."""
 
         return Utilities(self)
 
@@ -123,8 +109,34 @@ class Script(default.Script):
         - event: the Event
         """
 
+        length = event.source.queryText().characterCount
+        if event.detail1 == 0 and event.detail2 == length:
+            # seems to be generated by a reformat (ctrl+shift+f)
+            # or by commenting some block (ctrl+/).
+            # if not discarded, orca will speak all the text of the file.
+            return
+
         # Let the default script's normal behavior do its thing
         #
         default.Script.onTextInserted(self, event)
+        self._saveLastTextPosition(event.source)
+
+    def onTextDeleted(self, event):
+        """Called whenever text is deleted from an object.  Overridden here
+        so that we can avoid speaking text when caret moves after new text
+        is deleted.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # Let the default script's normal behavior do its thing
+        #
+        default.Script.onTextDeleted(self, event)
+        self._saveLastTextPosition(event.source)
+
+    def _saveLastTextPosition(self, obj):
+        if self.utilities.isTextArea(obj):
+            self._saveLastCursorPosition(obj, obj.queryText().caretOffset)
+
 
-        self._textInserted = (event.source.getRole() == pyatspi.ROLE_TEXT)
